@@ -9,12 +9,12 @@
 use blades::*;
 
 use beef::lean::Cow;
-use clap::Parser as ClapParser;
 use ramhorns::{Content, Ramhorns, Template};
 use serde::Deserialize;
 use serde_cmd::CmdBorrowed;
 
-use std::env::var;
+use std::collections::BTreeSet;
+use std::env;
 use std::ffi::OsStr;
 use std::fs::{self, File};
 use std::io::{self, stdin, stdout, BufRead, BufReader, BufWriter, ErrorKind, Lines, Write};
@@ -24,35 +24,39 @@ use std::time::{Instant, SystemTime};
 use std::{cmp, thread};
 use thiserror::Error;
 
+static HELP: &str = r#"Blazing fast dead simple static site generator
+
+Usage: blades [COMMAND]
+
+Commands:
+  init      Initialize the site in the current directory, creating the basic files and folders
+  new       Create a new page
+  build     Build the site according to config, content, templates and themes in the current directory
+  colocate  Move assets from the "assets" directory and from the theme, if one is used, into the output directory
+  all       Build the site and colocate the assets
+  lazy      Build the site and (colocate assets only if the theme was switched) [default]
+  help      Print this message
+  version   Print version information
+
+Environment variables:
+  BLADES_CONFIG   File to read the site config from [default: Blades.toml]
+"#;
+static VAR_CONFIG: &str = "BLADES_CONFIG";
 static CONFIG_FILE: &str = "Blades.toml";
+
 const BUFFER_SIZE: usize = 16384;
 
-#[derive(clap_derive::Parser)]
-#[clap(version, about)]
-/// Blazing fast Dead simple Static site generator
-struct Opt {
-    /// File to read the site config from
-    #[clap(short, long, default_value = CONFIG_FILE)]
-    config: String,
-    #[clap(subcommand)]
-    cmd: Option<Cmd>,
-}
-
-#[derive(PartialEq, clap_derive::Subcommand)]
+#[derive(PartialEq, Eq)]
 enum Cmd {
-    /// Initialise the site in the current directory, creating the basic files and folders
     Init,
-    /// Start creating a new page
     New,
-    /// Build the site according to config, content, templates and themes in the current directory
     Build,
-    /// Move the assets from the "assets" directory and from the theme, if one is used,
-    /// into the output directory
     Colocate,
-    /// Build the site and colocate the assets
     All,
-    /// Build the site and (colocate assets only if the theme was switched) [default]
     Lazy,
+    Help,
+    Version,
+    Invalid,
 }
 
 /// Main configuration where all the site settings are set.
@@ -149,7 +153,9 @@ struct MockPage {
 }
 
 /// Possible formats of the source.
+#[derive(Default)]
 enum Format {
+    #[default]
     Toml,
     Markdown,
 }
@@ -202,12 +208,6 @@ impl OutputResult for Output {
     }
 }
 
-impl Default for Format {
-    fn default() -> Self {
-        Format::Toml
-    }
-}
-
 impl Parser for Format {
     type Error = ParseError;
 
@@ -306,9 +306,34 @@ fn init() -> Result<(), Error> {
     let author = next_line(&mut lines, "Author:")?;
     let config = MockConfig { title, author };
     Template::new(include_str!("templates/Blades.toml"))?.render_to_file(CONFIG_FILE, &config)?;
-    fs::write(".watch.toml", include_str!("templates/.watch.toml"))?;
     fs::create_dir_all("content")?;
-    fs::create_dir_all("themes").map_err(Into::into)
+    let thm = next_line(&mut lines, "Start with a minimal working template? (Select no to use a theme; then put the theme inside the 'themes' directory.) [Y/n]")?;
+    if &thm == "n" || &thm == "N" || &thm == "no" || &thm == "No" {
+        fs::create_dir_all("themes").map_err(Into::into)
+    } else {
+        fs::create_dir_all("templates")?;
+        fs::write(
+            "templates/page.html",
+            include_str!("../examples/templates/page.html"),
+        )?;
+        fs::write(
+            "templates/section.html",
+            include_str!("../examples/templates/section.html"),
+        )?;
+        fs::write(
+            "templates/taxonomy.html",
+            include_str!("../examples/templates/taxonomy.html"),
+        )?;
+        fs::write(
+            "templates/taxonomy_key.html",
+            include_str!("../examples/templates/taxonomy_key.html"),
+        )?;
+        fs::write(
+            "templates/gallery.html",
+            include_str!("../examples/templates/gallery.html"),
+        )?;
+        Ok(())
+    }
 }
 
 /// Create a new page and edit it if the EDITOR variable is set
@@ -344,7 +369,7 @@ fn new_page(config: &Config) -> Result<(), Error> {
     Template::new(include_str!("templates/page.toml"))?.render_to_file(&path, &page)?;
     println!("{:?} created", &path);
 
-    if let Ok(editor) = var("EDITOR") {
+    if let Ok(editor) = env::var("EDITOR") {
         Command::new(editor).arg(&path).status()?;
     } else {
         println!("Set the EDITOR environment variable to edit new pages immediately");
@@ -413,11 +438,18 @@ fn load_templates(config: &Config) -> Result<Ramhorns, ramhorns::Error> {
 
 /// Delete all the pages that were present in the previous render, but not the current one.
 /// Then, write all the paths that were rendered to the file `filelist`
-fn cleanup(mut rendered: HashMap<PathBuf, u32>, filelist: &str) -> Result<(), io::Error> {
+fn cleanup(mut rendered: Vec<PathBuf>, filelist: &str) -> Result<(), io::Error> {
+    let mut set = BTreeSet::new();
+    for path in rendered.drain(..) {
+        if let Some(p) = set.replace(path) {
+            println!("Warning: more paths render to {}", p.to_string_lossy());
+        }
+    }
+
     if let Ok(f) = File::open(filelist) {
         BufReader::new(f).lines().try_for_each(|filename| {
             let filename = filename?;
-            if !rendered.contains_key(Path::new(&filename)) {
+            if !set.contains(Path::new(&filename)) {
                 // Every directory has its index rendered
                 if let Some(dir) = filename.strip_suffix("index.html") {
                     if dir.ends_with(path::is_separator) {
@@ -441,18 +473,33 @@ fn cleanup(mut rendered: HashMap<PathBuf, u32>, filelist: &str) -> Result<(), io
 
     let f = File::create(filelist)?;
     let mut f = BufWriter::new(f);
-    for (path, count) in rendered.drain() {
+    for path in rendered.drain(..) {
         // It was already checked that the paths contain valid UTF-8
         let path = path.into_os_string().into_string().unwrap();
         writeln!(&mut f, "{}", path)?;
-        if count > 1 {
-            println!("{} paths render to {}", count, path);
-        }
     }
 
     Ok(())
 }
 
+/// A helper trait to print the name of the file that was not found.
+trait PrintIfNotFound {
+    /// Print the name of the file that was not found.
+    fn print_if_not_found(self, name: &str) -> Self;
+}
+
+impl<T> PrintIfNotFound for io::Result<T> {
+    /// Print the name of the file that was not found.
+    fn print_if_not_found(self, name: &str) -> Self {
+        if let Err(ref e) = self {
+            if e.kind() == ErrorKind::NotFound {
+                eprintln!("Error: {} not found", name)
+            }
+        }
+        self
+    }
+}
+
 /// The actual logic of task parallelisation.
 fn build(config: &Config) -> Result<(), Error> {
     const MIN_PER_THREAD: usize = 5;
@@ -489,7 +536,12 @@ fn build(config: &Config) -> Result<(), Error> {
         .plugins
         .input
         .iter()
-        .map(|cmd| cmd.make_command().output()?.output_result(&cmd.path))
+        .map(|cmd| {
+            cmd.make_command()
+                .output()
+                .print_if_not_found(&cmd.path)?
+                .output_result(&cmd.path)
+        })
         .collect::<Result<Vec<_>, _>>()?;
     let input_pages = inputs
         .iter()
@@ -506,7 +558,8 @@ fn build(config: &Config) -> Result<(), Error> {
             .stdin(Stdio::piped())
             .stdout(Stdio::piped())
             .stderr(Stdio::piped())
-            .spawn()?;
+            .spawn()
+            .print_if_not_found(&cmd.path)?;
         let mut stdin = child.stdin.take().expect("Failed to open child stdin");
         if let Some(ref source) = transformed {
             stdin.write_all(source)?;
@@ -532,7 +585,8 @@ fn build(config: &Config) -> Result<(), Error> {
                     .stdin(Stdio::piped())
                     .stdout(Stdio::piped())
                     .stderr(Stdio::piped())
-                    .spawn()?;
+                    .spawn()
+                    .print_if_not_found(&config.plugins.content[cmd].path)?;
                 let mut stdin = child.stdin.take().expect("Failed to open child stdin");
                 if let Some(ref out) = output {
                     stdin.write_all(out.as_ref())?;
@@ -574,7 +628,7 @@ fn build(config: &Config) -> Result<(), Error> {
         let mut threads = Vec::with_capacity(num_threads);
         for chunk in pages.chunks(per_thread) {
             threads.push(s.spawn(|| {
-                let mut rendered = HashMap::default();
+                let mut rendered = Vec::with_capacity(2 * per_thread);
                 let mut buffer = Vec::with_capacity(BUFFER_SIZE);
                 for page in chunk.iter() {
                     page.render(context, &mut rendered, &mut buffer)?;
@@ -583,7 +637,12 @@ fn build(config: &Config) -> Result<(), Error> {
             }));
         }
 
-        let mut rendered = HashMap::default();
+        let tax_count = taxonomies.len()
+            + taxonomies
+                .iter()
+                .map(|(_, t)| t.keys().len())
+                .sum::<usize>();
+        let mut rendered = Vec::with_capacity(tax_count + 2 * pages.len());
         let mut buffer = Vec::with_capacity(BUFFER_SIZE);
         for (_, taxonomy) in taxonomies.iter() {
             taxonomy.render(context, &mut rendered, &mut buffer)?;
@@ -595,12 +654,7 @@ fn build(config: &Config) -> Result<(), Error> {
 
         for thread in threads.drain(..) {
             let mut other = thread.join().unwind()?;
-            for (path, count) in other.drain() {
-                rendered
-                    .entry(path)
-                    .and_modify(|c| *c += count)
-                    .or_default();
-            }
+            rendered.append(&mut other);
         }
         Ok::<_, Error>(rendered)
     })?;
@@ -615,7 +669,8 @@ fn build(config: &Config) -> Result<(), Error> {
                 .make_command()
                 .stdin(Stdio::piped())
                 .stderr(Stdio::piped())
-                .spawn()?;
+                .spawn()
+                .print_if_not_found(&cmd.path)?;
             let mut stdin = child.stdin.take().expect("Failed to open child stdin");
             stdin.write_all(pagedata.as_ref())?;
             drop(stdin);
@@ -628,41 +683,82 @@ fn build(config: &Config) -> Result<(), Error> {
     Ok(())
 }
 
+fn get_command() -> Cmd {
+    let mut args = env::args().skip(1);
+    let command = match args.next().as_deref() {
+        Some("init") => Cmd::Init,
+        Some("new") => Cmd::New,
+        Some("build") => Cmd::Build,
+        Some("colocate") => Cmd::Colocate,
+        Some("all") => Cmd::All,
+        Some("lazy") | None => Cmd::Lazy,
+        Some("help") => Cmd::Help,
+        Some("version") => Cmd::Version,
+        _ => Cmd::Invalid,
+    };
+    if args.next().is_some() {
+        Cmd::Invalid
+    } else {
+        command
+    }
+}
+
 fn main() {
-    let opt: Opt = ClapParser::parse();
+    let cmd = get_command();
+    let config_name: Cow<str> = env::var(VAR_CONFIG)
+        .map(Into::into)
+        .unwrap_or_else(|_| CONFIG_FILE.into());
+
     let start = Instant::now();
 
-    let config_file = match std::fs::read_to_string(&opt.config) {
+    match cmd {
+        Cmd::Help => {
+            print!("{}", HELP);
+            return;
+        }
+        Cmd::Version => {
+            println!("{} {}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"));
+            return;
+        }
+        Cmd::Invalid => {
+            eprintln!("Error: invalid arguments");
+            print!("{}", HELP);
+            return;
+        }
+        _ => {}
+    }
+
+    let config_file = match std::fs::read_to_string(config_name.as_ref()) {
         Ok(cf) => cf,
         // Don't need a config file for initialisation.
-        Err(_) if opt.cmd == Some(Cmd::Init) => "".to_string(),
+        Err(_) if cmd == Cmd::Init => "".to_string(),
         Err(e) => {
-            eprintln!("Can't read {}: {}", &opt.config, e);
+            eprintln!("Can't read {}: {}", config_name, e);
             return;
         }
     };
     let config: Config = match toml::from_str(&config_file) {
         Ok(cfg) => cfg,
         Err(e) => {
-            eprintln!("Error parsing config file {}: {}", &opt.config, e);
+            eprintln!("Error parsing config file {}: {}", config_name, e);
             return;
         }
     };
 
-    if let Err(e) = match opt.cmd {
-        Some(Cmd::Init) => {
+    if let Err(e) = match cmd {
+        Cmd::Init => {
             if config_file.is_empty() {
                 init()
             } else {
-                println!("Config file {} already present; exiting", &opt.config);
+                println!("Config file {} already present; exiting", &config_name);
                 Ok(())
             }
         }
-        Some(Cmd::New) => new_page(&config),
-        Some(Cmd::Build) => build(&config),
-        Some(Cmd::Colocate) => colocate_assets(&config).map_err(Into::into),
-        Some(Cmd::All) => build(&config).and_then(|_| colocate_assets(&config).map_err(Into::into)),
-        Some(Cmd::Lazy) | None => build(&config).and_then(|_| {
+        Cmd::New => new_page(&config),
+        Cmd::Build => build(&config),
+        Cmd::Colocate => colocate_assets(&config).map_err(Into::into),
+        Cmd::All => build(&config).and_then(|_| colocate_assets(&config).map_err(Into::into)),
+        Cmd::Lazy => build(&config).and_then(|_| {
             if fs::read_to_string(OLD_THEME)
                 .map(|old| old != config.theme)
                 .unwrap_or(true)
@@ -673,9 +769,13 @@ fn main() {
                 Ok(())
             }
         }),
+        _ => {
+            unreachable!()
+        }
     } {
-        eprintln!("{}", e)
-    } else if !(opt.cmd == Some(Cmd::Init) || opt.cmd == Some(Cmd::New)) {
-        println!("Done in {}ms.", start.elapsed().as_micros() as f64 / 1000.0)
+        eprintln!("{}", e);
+        return;
     }
+
+    println!("Done in {}ms.", start.elapsed().as_micros() as f64 / 1000.0)
 }
