@@ -13,6 +13,7 @@ use crate::taxonomies::{Classification, Taxonomies};
 use crate::types::{Ancestors, Any, DateTime, HashMap};
 
 use beef::lean::Cow;
+use chrono::NaiveDate;
 use ramhorns::{
     encoding::Encoder, traits::ContentSequence, Content, Error, Ramhorns, Section, Template,
 };
@@ -272,7 +273,7 @@ pub(crate) trait Paginate: Content + Sized {
         by: usize,
         path: &mut PathBuf,
         tpl: &Template,
-        rendered: &mut HashMap<PathBuf, u32>,
+        rendered: &mut Vec<PathBuf>,
         buffer: &mut Vec<u8>,
     ) -> Result<(), io::Error> {
         let (mut first, last) = (range.start, range.end);
@@ -322,7 +323,18 @@ impl<'p> Page<'p> {
         page.pages = source.pages.clone();
         page.subsections = source.subsections.clone();
         page.parent = source.parent;
-        page.date = page.date.or_else(|| source.date.map(|d| d.into()));
+
+        let slug = path.rsplit(is_separator).next().unwrap_or_default();
+        page.date = page
+            .date
+            .or_else(|| {
+                slug.get(..10).and_then(|p| {
+                    p.parse::<NaiveDate>()
+                        .map(|d| DateTime(d.and_hms_opt(0, 0, 0).unwrap()))
+                        .ok()
+                })
+            })
+            .or_else(|| source.date.map(|d| d.into()));
 
         if is_section || page.slug.is_empty() || page.slug.contains(is_separator) {
             let slug = path.rsplit(is_separator).next().unwrap_or_default();
@@ -410,7 +422,7 @@ impl<'p> Page<'p> {
     pub fn render(
         &self,
         Context(all, site, classification, templates, output_dir): Context<'p, '_>,
-        rendered: &mut HashMap<PathBuf, u32>,
+        rendered: &mut Vec<PathBuf>,
         buffer: &mut Vec<u8>,
     ) -> Result<(), Error> {
         let mut output = output_dir.join(self.path.as_ref());
@@ -613,7 +625,7 @@ impl<'p> Pages<'p> {
     }
 }
 
-impl<'p> Ord for Page<'p> {
+impl Ord for Page<'_> {
     fn cmp(&self, other: &Self) -> Ordering {
         let mut self_paths = self.path.0.split(is_separator);
         let mut other_paths = other.path.0.split(is_separator);
@@ -638,21 +650,21 @@ impl<'p> Ord for Page<'p> {
     }
 }
 
-impl<'p> PartialOrd for Page<'p> {
+impl PartialOrd for Page<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
     }
 }
 
-impl<'p> PartialEq for Page<'p> {
+impl PartialEq for Page<'_> {
     fn eq(&self, other: &Self) -> bool {
         self.path.0 == other.path.0 && self.slug == other.slug && self.id == other.id
     }
 }
 
-impl<'p> Eq for Page<'p> {}
+impl Eq for Page<'_> {}
 
-impl<'p, 'r> Paginate for PageContext<'p, 'r> {
+impl Paginate for PageContext<'_, '_> {
     #[inline]
     fn paginate(&self, pages: Range<usize>, length: usize, current: usize) -> Self {
         let old = &self.pages;
@@ -704,12 +716,22 @@ impl<'p, 'r> PageList<'p, 'r> {
 
 #[inline]
 fn render_content<E: Encoder>(source: &str, encoder: &mut E) -> Result<(), E::Error> {
-    let parser = pulldown_cmark::Parser::new_ext(source, pulldown_cmark::Options::all());
+    use pulldown_cmark::Options;
+
+    let options = Options::ENABLE_TABLES
+        | Options::ENABLE_FOOTNOTES
+        | Options::ENABLE_STRIKETHROUGH
+        | Options::ENABLE_TASKLISTS
+        | Options::ENABLE_SMART_PUNCTUATION
+        | Options::ENABLE_HEADING_ATTRIBUTES
+        | Options::ENABLE_MATH
+        | Options::ENABLE_GFM;
+    let parser = pulldown_cmark::Parser::new_ext(source, options);
     let processed = cmark_syntax::SyntaxPreprocessor::new(parser);
     encoder.write_html(processed)
 }
 
-impl<'p, 'r> Content for PageList<'p, 'r> {
+impl Content for PageList<'_, '_> {
     #[inline]
     fn is_truthy(&self) -> bool {
         !self.range.is_empty()
@@ -731,7 +753,7 @@ impl<'p, 'r> Content for PageList<'p, 'r> {
     }
 }
 
-impl<'p, 'r> Content for Permalink<'p, 'r> {
+impl Content for Permalink<'_, '_> {
     #[inline]
     fn is_truthy(&self) -> bool {
         true
@@ -768,7 +790,7 @@ impl<'p, 'r> Content for Permalink<'p, 'r> {
     }
 }
 
-impl<'p, 'r> Content for Pictures<'p, 'r> {
+impl Content for Pictures<'_, '_> {
     #[inline]
     fn is_truthy(&self) -> bool {
         !self.0.is_empty()
@@ -790,7 +812,7 @@ impl<'p, 'r> Content for Pictures<'p, 'r> {
     }
 }
 
-impl<'p, 'r> Content for PicturePermalink<'p, 'r> {
+impl Content for PicturePermalink<'_, '_> {
     #[inline]
     fn is_truthy(&self) -> bool {
         true
@@ -831,7 +853,7 @@ impl<'p> Deref for Pages<'p> {
     }
 }
 
-impl<'p> DerefMut for Pages<'p> {
+impl DerefMut for Pages<'_> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.0
     }
